;; -*- mode: scheme -*-
;;
;; Gauche interactive session with readline support.
;;
;; By: YOKOTA Hiroshi

(use term.readline)
(use gauche.interactive)
(use srfi-1) ; for (append-map)

;; useful functions from "gauche.interactive" (info, describe, apropos)
(define (/i obj)
  (info (cond ((subr?      obj) (procedure-info obj))
	      ((procedure? obj) (procedure-info obj))
	      (else             obj))))
(define /d describe)
(define (/a obj)
  (apropos (if (string? obj)
	       (string->regexp obj)
	       obj)))

(define (main args)
  (let ((readline-port (make-readline-input-port))
	(histfile (string-append (or (sys-getenv "HOME") ".")
				 "/.gosh_history")))
    (define (all-functions-list)
      (append-map (lambda (mod)
		    (hash-table-map (module-table mod)
				    (lambda (sym gloc) (symbol->string sym))))
		  (all-modules)))

    (rl-set-application-name!            "Gauche")
    (rl-set-basic-word-break-characters! " \t\n()',\";")
    (rl-set-basic-quote-characters!      "\"")
    (rl-set-prompt!                      :ps1 "gosh> " :ps2 "..... ")
    (rl-set-completion-list-function!    (lambda () (all-functions-list)))
    (rl-stifle-history!                  1000)

    (dynamic-wind
        (lambda ()
	  (rl-read-history! histfile))
	(lambda ()
	  (read-eval-print-loop (lambda ()
				  (rl-set-prompt-state! 'ps1)
				  (read readline-port))
				#f
				#f
				(lambda () (rl-commit-history!))))
	(lambda ()
	  (rl-write-history histfile)))

    0))

;; end
